function y=nt_inpaint(x,w,w1)
%y=nt_inpaint(x,w,cw1) - interpolate missing samples of matrix
%
%  y: interpolated data
%
%  x: data to interpolate (time*channels)
%  w: weight matrix to control interpolation (same size as x)
%  w1: weight matrix to calculate covariance (if missing use min(w,2))
%
% The missing parts of each channel x(:,i) (flagged by w(:,i)) are replaced
% from intact channels, using projection matrix calculated from w1.
%
%  NoiseTools
% See nt_regw, nt_detrend_robust, nt_star2.
%

nt_greetings;
PCA_THRESH=0.000000001;

if nargin<2; error('!'); end
if nargin<3; w1=[]; end

if nargout==0;
    % don't return result, just plot
    y=nt_inpaint(x,w,c);
    figure 1; clf;
    subplot 211; 
    plot(nt_unfold(x)); title('raw');
    subplot 212; 
    plot(nt_unfold(y)); title('processed');
    clear y; return
end

%% check/fix sizes
if any(size(w) ~= size(x)); error('w must be of same size as x'); end
if any(w ~= 0 & w ~= 1); error('weights must be 0 or 1'); end
m=size(x,1);
x=nt_unfold(x);
w=nt_unfold(w);
iSkipChannels=find(all(w==0));
if ~isempty (iSkipChannels); 
    disp(['ignore channels marked as bad: ', num2str(iSkipChannels)]);
    x0=x;
    x(:,iSkipChannels)=[];
    w(:,iSkipChannels)=[];
end
if ~isempty(w1)
    if size(w1,2)~=1; 
        disp(size(w1,2)); 
        error('!'); 
    end
else
    disp('calculating covariance based on weight matrix');
    w1=min(w,[],2);
    propGood=mean(w1);
    disp(['proportion good samples: ',num2str(propGood)]);
    if ~propGood;
        error('no good samples')
    end
end
y=x;
x=nt_demean(x,w1);
mn=x-y; % save mean
s=sum(bsxfun(@times, x, w1));
c=nt_cov(bsxfun(@times, x, w1))/sum(w1);
disp([sum(w1), numel(find(isnan(c)))]);


%% find different patterns of weights, merge to reduce computation
[wDict,nOccurrences,iBack]=patternDict(w); 
disp(['distinct patterns, mean weight: ', num2str(size(wDict,1)), ', ', num2str(mean(w(:)))]);
[w,x]=coalesceWeights(w,x);
[w,x]=coalesceWeights(w,x);
[w,x]=coalesceWeights(w,x);
[wDict,nOccurrences,iBack]=patternDict(w); 
disp([' ... reduced to: ', num2str(size(wDict,1)), ', ', num2str(mean(w(:)))]);

for iPattern=1:size(wDict,1);
    iBad=find(~wDict(iPattern,:)); % channel indices
    iGood=find(wDict(iPattern,:)); 
    if isempty(iBad); continue; end
    iInPattern=find(iBack==iPattern); % sample indices
    iOutPattern=find(iBack~=iPattern);
    %ss=s-sum(bsxfun(@times, x(iInPattern,:), w1(iInPattern,:))); 
    %mm=ss/numel(iOutPattern);
    cc = c;% - mm'*mm;%*numel(iOutPattern);
    [V,D]=eig(cc(iGood,iGood)); V=real(V); D=real(D);
    topcs=V(:,find(D/max(D) > PCA_THRESH)); % discard weak dims
    b=cc(iBad,iGood)*topcs/(topcs'*cc(iGood,iGood)*topcs); % projection matrix  
    xx=(x(iInPattern,iGood));
    y(iInPattern,iBad)=xx*(topcs*b')-mn(iInPattern,iBad); % projection 
end

if ~isempty(iSkipChannels)
    yy=x0;
    yy(:,setdiff(1:size(x0,2),iSkipChannels))=y;
    y=yy;
end

%%% TEST %%%
if 0
    x0=sin(2*pi*(1:10000)'*(1:10)/10000);
    x=x0*randn(10,20)+10;
    w=ones(size(x));  
    w(1:4000,1)=0; w(5001:6000,3)=0; w(6000:6500,4)=0; w(7000:7500,5)=0; w(8000:8500,6)=0; w(9000:9500,7)=0;
    x(6000:6500,4)=0;x(7000:7500,5)=10;
    y=nt_inpaint(x,w);
    figure(2); clf
    subplot 211; plot([x(:,1),y(:,1)]); legend('original','processed'); legend boxoff
    subplot 212; plot([x(:,1)-y(:,1)]); legend('difference'); legend boxoff
end
if 0
    x0=sin(2*pi*(1:10000)'*(1:10)/10000);
    x=x0*randn(10,20)+10;
    w=ones(size(x));  
    w(1:4000,1)=0; 
    w(:,8)=0; 
    x(1:2000,1)=1;
    y=nt_inpaint(x,w);
    figure(2); clf
    subplot 211; plot([x(:,1),y(:,1)]); legend('original','processed'); legend boxoff
    subplot 212; plot([x(:,1)-y(:,1)]); legend('difference'); legend boxoff
end
if 0
    [p,x]=nt_read_data('/data/meg/theoldmanandthesea/eeg/mc/MC_aespa_speech_47.mat');
    x=x'; x=x(:,1:128); x=x(1:10000,:);
    x=nt_demean(x);
    [x,w]=nt_detrend_robust(x,10);
    y=nt_inpaint(x,w);
    figure(1); clf
    subplot 311; plot(x); subplot 312; plot(y); subplot 313; plot(x-y);
    figure(2); clf
    ch=35;subplot 311; plot([x(:,ch),y(:,ch)]); subplot 312; plot([x(:,ch)-y(:,ch)]); subplot 313; plot(w(:,ch), '.-');
end


function [ww,nOccurrences,iBack]=patternDict(w);
% ww: dictionary of patterns
% nOccurrences: number of times each pattern occurred
% iBack: index to reconstruct input from dictionary
[ww,~,IC,nOccurrences]=nt_unique(w,'rows');
[nOccurrences,iSort]=sort(nOccurrences, 'descend'); % sort by decreasing number
[~,iReverse]=sort(iSort); % 
ww=ww(iSort,:); % same order for patterns, w = ww(iReverse1(IC),:)
iBack=iReverse(IC); % w = ww(iBack,:)

function [w,x]=coalesceWeights(w,x)
% reduce the number of weight patterns based on pruning & interpolation, to
% save time
nchans=size(w,2);
w1=w(1:end-2,:); 
w2=w(2:end-1,:); 
w3=w(3:end,:); 
x1=x(1:end-2,:); 
x2=x(2:end-1,:); 
x3=x(3:end,:); 
% weights that correspond to a single sample 
idx=find(~w1&w2&~w3); % isolated good, set to bad
w2(idx)=0; w(2:end-1,:)=w2;
idx=find(w1&~w2&w3); % isolated bad, interpolate, set to good
w2(idx)=1; w(2:end-1,:)=w2; 
x2(idx)=(x1(idx))+x3(idx)/2; x(2:end-1,:)=x2;
% weight patterns that occur rarely
[wDict,nOccurrences,iBack]=patternDict(w); 
idx=find(nOccurrences<=3);
isolated=iBack(idx);
for k=1:numel(isolated)
    iIsolated=isolated(k);
    if iIsolated-1>=1 && iIsolated+1<=size(w,1)            
        if sum(abs(w(iIsolated-1,:)-w(iIsolated,:)))<sum(abs(w(iIsolated,:)-w(iIsolated+1,:)));
            iFix=iIsolated-1;% preceding is closest
        else
            iFix=iIsolated+1;% following is closest
        end
        idx=find(w(iIsolated,:)&~w(iFix,:));
        w(iIsolated,idx)=0;
        idx=find(~w(iIsolated,:)&w(iFix,:));
        w(iIsolated,idx)=1;
        x(iIsolated,idx)=(x(iIsolated-1,idx)+x(iIsolated+1,idx))/2;
    end
end

